@@ -91,11 +91,15 @@ reduced to the facility to terminate the inferior process.
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
-#if defined(HAVE_SYS_IOCTL_H) && !defined(__sun__) /* leads to redefines */
+#ifdef HAVE_SYS_IOCTL_H
+/* MacOS 15 (beta) sys/ioctl.h pulls in socket.h */
+#undef send
 #include <sys/ioctl.h>
+#define send sendPCE
 #endif
 
 #ifdef __WINDOWS__
+#include <msw/msprocess.h>
 #define environ _environ	/* declared in STDLIB.H */
 #else
 #ifdef HAVE__NSGETENVIRON
@@ -1114,4 +1118,3 @@ copyTty(Process p, char *pty, int fd)
 }
 
 #endif /*HAVE_PTYS*/
-
